/**
 * SessionsDBProvider.java: Content Provider to access practice sessions data.
 *            Copyright (C) 2012 donfede@casagrau.org
 *
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; version 2 of the License.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 59 Temple
 * Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * created 2012-10-13 donfede@casagrau.org
 * credit: to the Android SDK sample Content Provider apps, NotesList and SearchableDictionary 
 *
 * $Id: SessionsDBProvider.java,v 1.26 2012-11-06 00:43:53 donfede Exp $
 */
package org.casagrau.mpq.mpq_android;

import java.text.SimpleDateFormat;
import java.util.Date;

import android.content.ContentProvider;
import android.content.ContentUris;
import android.content.ContentValues;
import android.content.Context;
import android.content.UriMatcher;
import android.database.Cursor;
import android.database.SQLException;
import android.database.sqlite.SQLiteDatabase;
import android.database.sqlite.SQLiteOpenHelper;
import android.database.sqlite.SQLiteQueryBuilder;
import android.net.ParseException;
import android.net.Uri;
import android.provider.BaseColumns;
import android.util.Log;

/**
 * Provides access to a database of practice sessions and related data. 
 * Data "tables" include: Practices, Goals, Instruments
 * each with their own attributes.
 */
public class SessionsDBProvider extends ContentProvider {
    private static final String LOGTAG = "SessionsDBProvider";

    // DB constants
    private static final String DB_NAME = "mpq_practices.db";
    private static final int DB_VERSION = 13; // DB versions before 20 drop data between schema upgrades
    
    // Uri and UriMatcher constants -- FIXME: the UriMatcher feels warty ... should these go somewhere else?  Q-does anyone outside of this class need them... don't think so... maybe leave them for now (2012-10-21)
    public static final String AUTHORITY = "org.casagrau.mpq.mpq_android.SessionsDBProvider";

    private static final int URIM_PRACTICES = 0;
    private static final int URIM_PRACTICES_ID = 1;   
    private static final int URIM_PRACTICES_COUNT = 2;
    private static final int URIM_PRACTICES_PREV_ID = 3;
    private static final int URIM_PRACTICES_NEXT_ID = 4;
    private static final int URIM_GOALS = 5;
    private static final int URIM_GOALS_ID = 6;
    private static final int URIM_INSTRUMENTS = 7; 
    private static final int URIM_INSTRUMENTS_ID = 8;
    private static final UriMatcher sUriMatcher = buildUriMatcher();      // Creates a UriMatcher object.
    
    /**
     * Builds up a UriMatcher for various table queries.
     */
    private static UriMatcher buildUriMatcher() {
        UriMatcher matcher =  new UriMatcher(UriMatcher.NO_MATCH);
        // to get definitions...
        matcher.addURI(AUTHORITY, SessionsDB.PRACTICES_TABLE, URIM_PRACTICES);
        matcher.addURI(AUTHORITY, SessionsDB.PRACTICES_TABLE+"/#", URIM_PRACTICES_ID);
        matcher.addURI(AUTHORITY, SessionsDB.PRACTICES_TABLE_COUNT, URIM_PRACTICES_COUNT);
        matcher.addURI(AUTHORITY, SessionsDB.PRACTICES_TABLE_PREV+"/#", URIM_PRACTICES_PREV_ID);
        matcher.addURI(AUTHORITY, SessionsDB.PRACTICES_TABLE_NEXT+"/#", URIM_PRACTICES_NEXT_ID);
        matcher.addURI(AUTHORITY, SessionsDB.GOALS_TABLE, URIM_GOALS);
        matcher.addURI(AUTHORITY, SessionsDB.GOALS_TABLE+"/#", URIM_GOALS_ID);
        matcher.addURI(AUTHORITY, SessionsDB.INSTRUMENTS_TABLE, URIM_INSTRUMENTS);
        matcher.addURI(AUTHORITY, SessionsDB.INSTRUMENTS_TABLE+"/#", URIM_INSTRUMENTS_ID);
        return matcher;
    }

    
    //////////////////////////////////////////////
    // public (inner)CONTRACT CLASS DEFINING TABLE NAMES AND COLUMNS 
    /**
     * Contract Class defining elements to access the Sessions database, including URI, table name, and table column constants.
     * This class is a Contract Class only - it cannot be instantiated.
     */
    public static final class SessionsDB implements BaseColumns{
    	//This class cannot be instantiated (it's meant to be a "Contract Class" for the content provider.
    	private SessionsDB() {}

    	/**
    	 * Practice table name
    	 */
        public static final String PRACTICES_TABLE = "practices";
    	
        
        /**
         * Practices table count - a meta query, returning -- select count(*).
         */
        public static final String PRACTICES_TABLE_COUNT = "practices_count";
        
        /**
         * Practices table previous row _ID - a meta query, returning the previous available row ID.
         */
        public static final String PRACTICES_TABLE_PREV = "practices_prev";
        
        /**
         * Practices table next row _ID - a meta query, returning the next available row ID.
         */
        public static final String PRACTICES_TABLE_NEXT = "practices_next";
        
        /**
         * Practice date time stamp
         * <P>Type: INTEGER (UNIX Time)</P>
         */
        public static final String PRACTICE_DATE_STAMP = "practice_date_stamp";
        
        /**
         * Practice length total
         * <P>Type: INTEGER (seconds)
         */
        public static final String PRACTICE_LENGTH_TOTAL = "practice_length_total";
        
        /**
         * Practice length noise
         * <P>Type: INTEGER (seconds)
         */
         public static final String PRACTICE_LENGTH_NOISE = "practice_length_noise";
         
        /**
         * Practice audio file
         * <P>Type: TEXT</P>
         */
         public static final String PRACTICE_AUDIO_FILE = "practice_audio_file";
        
        /**
         * Practice goal
         * <P>Type: TEXT</P>
         */
        public static final String PRACTICE_GOAL_ID = "practice_goal_id";

        /**
         * Practice notes
         * <P>Type: TEXT</P>
         */
        public static final String PRACTICE_NOTES = "practice_notes";

        /**
         * Practice instrument
         * <P>Type: TEXT</P>
         */
        public static final String PRACTICE_INSTRUMENT_ID = "practice_instrument_id";
    	
    	/**
    	 * Goal table name
    	 */
        public static final String GOALS_TABLE = "goals";
    	
        /**
         * Goal name
         * <P>Type: TEXT</P>
         */
        public static final String GOAL_NAME = "goal_name";

        /**
         * Goal description
         * <P>Type: TEXT</P>
         */
        public static final String GOAL_DESC = "goal_desc";
        
        /**
         * Instrument table name
         */
        public static final String INSTRUMENTS_TABLE = "instruments";
        
        /**
         * Instrument name
         */
        public static final String INSTRUMENT_NAME = "inst_name";
        
        /**
         * Instrument description
         */
        public static final String INSTRUMENT_DESC = "inst_desc";

        
        /**
         * URI Constants
         */
    	public static final Uri CONTENT_PRACTICES_URI = Uri.parse("content://" + AUTHORITY + "/" + PRACTICES_TABLE);
    	public static final Uri CONTENT_PRACTICES_COUNT_URI = Uri.parse("content://" + AUTHORITY + "/" + PRACTICES_TABLE_COUNT);
    	public static final Uri CONTENT_PRACTICES_PREV_URI = Uri.parse("content://" + AUTHORITY + "/" + PRACTICES_TABLE_PREV);
    	public static final Uri CONTENT_PRACTICES_NEXT_URI = Uri.parse("content://" + AUTHORITY + "/" + PRACTICES_TABLE_NEXT);    	
    	public static final Uri CONTENT_GOALS_URI = Uri.parse("content://" + AUTHORITY + "/" + GOALS_TABLE);  
        public static final Uri CONTENT_INSTRUMENTS_URI = Uri.parse("content://" + AUTHORITY + "/" + INSTRUMENTS_TABLE);
        
        
        /**
         * Static helper methods, to provide consistency (ex: with data formatting)
         */

    	// ///////////////////////////////////////////////////
    	// Helper methods.
    	/**
    	 * Convert date in long int milliseconds format into ISO 8601 format.
    	 */
    	public static String longDateString(long dateSec) {
    		Log.d(LOGTAG, "longDateStringLoader() 00 dateSet: " + dateSec);
    		String lStr = "";
    		SimpleDateFormat iso8601Format = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss"); 		// FIXME: this method will get called frequently -- Formater maybe should be static, or at the object level
    		try {
    			lStr = lStr + iso8601Format.format(new Date(dateSec));						
    		} catch (ParseException e) {
    			Log.e(LOGTAG, "longDateString - Parsing ISO8601 datetime failed", e);
    		}
    		return lStr;
    	} // END longDateString()
    	
    	/**
    	 * Convert session in milliseconds format into HH:mm:ss format. 
    	 */
    	public static String longSessionMSecString(long sessionMSec) {
    		Log.d(LOGTAG, "intSessionSecString() 00 sessionSec: " + sessionMSec);
    		String lStr = "";
    		
    		/** // requires API level >= 9
//    		lStr = String.format("%d min, %d sec", 
//    			    TimeUnit.MILLISECONDS.toMinutes(sessionMSec),
//    			    TimeUnit.MILLISECONDS.toSeconds(sessionMSec) - 
//    			    TimeUnit.MINUTES.toSeconds(TimeUnit.MILLISECONDS.toMinutes(sessionMSec))
//    			); 
    		 */	
    		int lseconds = (int) (sessionMSec / 1000) % 60 ;
    		int lminutes = (int) ((sessionMSec / (1000*60)) % 60);
    		int lhours   = (int) ((sessionMSec / (1000*60*60)) % 24);
    		lStr = String.format("%02d:%02d:%02d",
    				lhours, lminutes, lseconds);				
    		
    		return lStr;
    	} // END intSessoinSecString()       
    	
    	
    	/**
    	 * Convert date in string ISO 8601 format back to long int milliseconds.
    	 */
    	public static long strDateLong(String dateStr) {
    		Log.d(LOGTAG, "strDateLong() 00 dateStr: " + dateStr);
    		long ldateLong = 0;
    		Date ldate;
    		SimpleDateFormat iso8601Format = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss"); 		// FIXME: this method will get called frequently -- Formater maybe should be static, or at the object level
    		try {
    			ldate =  iso8601Format.parse(dateStr);
    			ldateLong = ldate.getTime();
    		} catch (java.text.ParseException e) {
    			Log.e(LOGTAG, "strDateLong - Parsing ISO8601 datetime failed", e);
    		}
    		Log.d(LOGTAG, "strDateLong() 99 dateLong: " + ldateLong);
    		return ldateLong;
    	} // END strDateLong()
    	
    	/**
    	 * Convert session in HH:mm:ss format into milliseconds. 
    	 */
    	public static long strSessionMSecLong(String sessionStr) {
    		Log.d(LOGTAG, "strSessionMSecLong() 00 sessionStr: " + sessionStr);
    		long ldateLong = 0;
    		Date ldate;
    		String lStr = "";
    		SimpleDateFormat sessionFormat = new SimpleDateFormat("HH:mm:ss"); 		// FIXME: this method will get called frequently -- Formater maybe should be static, or at the object level

    		try {
    			ldate =  sessionFormat.parse(sessionStr);
    			ldateLong = ldate.getTime();
    		} catch (java.text.ParseException e) {
    			Log.e(LOGTAG, "strDateLong - Parsing ISO8601 datetime failed", e);
    		}
    		Log.d(LOGTAG, "strSessionMSecLong() 99 dateLong: " + ldateLong);
    		return ldateLong;
    	} // END strSessionMSecLong()       
    	
    	
    } // END SessionDB Contract Class
    
    
    private DBHelper mDBHelper;
    //////////////////////////////////////////////
    // DBHelper code creating and managing the database.
    /**
     * This class helps open, create, and upgrade the database file.
     */
    private static class DBHelper extends SQLiteOpenHelper {

        DBHelper(Context context) {
            super(context, DB_NAME, null, DB_VERSION);
    		Log.d(LOGTAG, "DBHelper constructor");
        }

        @Override
        public void onCreate(SQLiteDatabase db) {
    		Log.d(LOGTAG, "DBHelper onCreate() 00");
        	// Create tables for practices, goals, and instruments
    		// Practices table
        	final String SQL_CREATE_PRACTICES = "CREATE TABLE " + SessionsDB.PRACTICES_TABLE +
        			"( " + SessionsDB._ID + " INTEGER PRIMARY KEY, " +
        			SessionsDB.PRACTICE_DATE_STAMP + " INTEGER, " + 
        			SessionsDB.PRACTICE_LENGTH_TOTAL + " INTEGER, " +
        			SessionsDB.PRACTICE_LENGTH_NOISE + " INTEGER, " +
        			SessionsDB.PRACTICE_AUDIO_FILE + " TEXT," +
        			SessionsDB.PRACTICE_GOAL_ID + " INTEGER, " + 
        			SessionsDB.PRACTICE_NOTES + " TEXT, " +
        			SessionsDB.PRACTICE_INSTRUMENT_ID + " INTEGER );" ;
            Log.d(LOGTAG, "DBHelper.onCreate() 10 SQL pract: " + SQL_CREATE_PRACTICES);
        	db.execSQL(SQL_CREATE_PRACTICES);
            
        	final String SQL_CREATE_GOALS = "CREATE TABLE " + SessionsDB.GOALS_TABLE +
        			"( " + SessionsDB._ID + " INTEGER PRIMARY KEY, " +
        			SessionsDB.GOAL_NAME + " TEXT, " +
        			SessionsDB.GOAL_DESC + " TEXT );" ;
            Log.d(LOGTAG, "DBHelper.onCreate() 20 SQL goal: " + SQL_CREATE_GOALS);
        	db.execSQL(SQL_CREATE_GOALS);
        	
        	final String SQL_CREATE_INSTRUMENTS = "CREATE TABLE " + SessionsDB.INSTRUMENTS_TABLE +
        			"( " + SessionsDB._ID + " INTEGER PRIMARY KEY, " +
        			SessionsDB.INSTRUMENT_NAME + " TEXT, " +
        			SessionsDB.INSTRUMENT_DESC + " TEXT );" ;
            Log.d(LOGTAG, "DBHelper.onCreate() 20 SQL instruments: " + SQL_CREATE_INSTRUMENTS);
        	db.execSQL(SQL_CREATE_INSTRUMENTS);


            // Insert some sample data.
            ContentValues val = new ContentValues();
            // goals dummy data
            val.put(SessionsDB.GOAL_NAME, "goal1 - blues");
            val.put(SessionsDB.GOAL_DESC, "That thing when you puffin");
            db.insert(SessionsDB.GOALS_TABLE, null, val);            			
            val.put(SessionsDB.GOAL_NAME, "goal2 - simandl");
            val.put(SessionsDB.GOAL_DESC, "Get at least 15 minutes on the instrument - but what does that mean");
            db.insert(SessionsDB.GOALS_TABLE, null, val);
            // instruments dummy data
            val.clear();
            val.put(SessionsDB.INSTRUMENT_NAME, "piano");
            val.put(SessionsDB.INSTRUMENT_DESC, "crummy keyboard");
            db.insert(SessionsDB.INSTRUMENTS_TABLE, null, val);            			
            val.put(SessionsDB.INSTRUMENT_NAME, "kazoo");
            val.put(SessionsDB.INSTRUMENT_DESC, "loud and obnoxious, but portable");
            db.insert(SessionsDB.INSTRUMENTS_TABLE, null, val);   
            // practice sessions data
            val.clear();
            val.put(SessionsDB.PRACTICE_DATE_STAMP, java.lang.System.currentTimeMillis()); // insert current time - this is test data
            val.put(SessionsDB.PRACTICE_LENGTH_TOTAL, 903000);
            val.put(SessionsDB.PRACTICE_LENGTH_NOISE, 610000);
            val.put(SessionsDB.PRACTICE_AUDIO_FILE, "/mpq/file1.3gp");
            val.put(SessionsDB.PRACTICE_GOAL_ID, 1);
            val.put(SessionsDB.PRACTICE_NOTES, "running along, chasing fish");
            val.put(SessionsDB.PRACTICE_INSTRUMENT_ID, 1);
            db.insert(SessionsDB.PRACTICES_TABLE, null, val);
            val.put(SessionsDB.PRACTICE_DATE_STAMP, java.lang.System.currentTimeMillis()); // insert current time - this is test data
            val.put(SessionsDB.PRACTICE_LENGTH_TOTAL, 1325000);
            val.put(SessionsDB.PRACTICE_LENGTH_NOISE, 941000);
            val.put(SessionsDB.PRACTICE_AUDIO_FILE, "/mpq/file2.3gp");
            val.put(SessionsDB.PRACTICE_GOAL_ID, 2);
            val.put(SessionsDB.PRACTICE_NOTES, "did more than 15");
            val.put(SessionsDB.PRACTICE_INSTRUMENT_ID, 2);
            db.insert(SessionsDB.PRACTICES_TABLE, null, val);
            val.clear();
            val.put(SessionsDB.PRACTICE_DATE_STAMP, "13510376100"); 
            val.put(SessionsDB.PRACTICE_LENGTH_TOTAL, 1235000);
            val.put(SessionsDB.PRACTICE_LENGTH_NOISE, 541000);
            val.put(SessionsDB.PRACTICE_AUDIO_FILE, "/mpq/file3.3gp");
            val.put(SessionsDB.PRACTICE_GOAL_ID, 2);
            val.put(SessionsDB.PRACTICE_NOTES, "");
            val.put(SessionsDB.PRACTICE_INSTRUMENT_ID, 1);
            db.insert(SessionsDB.PRACTICES_TABLE, null, val);


            
    		Log.d(LOGTAG, "DBHelper onCreate() 99");
        }

        @Override
        public void onUpgrade(SQLiteDatabase db, int oldVersion, int newVersion) {
            Log.w(LOGTAG, "Upgrading database from version " + oldVersion + " to "
                    + newVersion + ", which will destroy all old data");
            if (oldVersion <= 20) { // abusively delete and recreate tables and data during early development versions FIXME: add code to evolve from one version to the next... big switch
            	db.execSQL("DROP TABLE IF EXISTS " + SessionsDB.PRACTICES_TABLE);
            	db.execSQL("DROP TABLE IF EXISTS " + SessionsDB.GOALS_TABLE);
            	db.execSQL("DROP TABLE IF EXISTS " + SessionsDB.INSTRUMENTS_TABLE);
            	onCreate(db);
            }
        }
    } // END DBHelper class
    

    
    ///////////////////
    // ContentProvider classes 
    /**
     * Initialize the database helper.
     */
	@Override
	public boolean onCreate() {
        mDBHelper = new DBHelper(getContext());
        return true;
	}

	/**
	 * General query handler, which dispatches the query as appropriate.
	 */
	@Override
	public Cursor query(Uri uri, String[] projection, String selection,
			String[] selectionArgs, String sortOrder) {
		Log.d(LOGTAG, "privder query() 00 uri:" + uri);

        // Use the UriMatcher to see what kind of query we have and format the db query accordingly
		int urimId = sUriMatcher.match(uri);
        switch (urimId) {
            case URIM_PRACTICES:
            case URIM_PRACTICES_ID:
        		Log.d(LOGTAG, "privder query() 20 practices");
        		return getGeneralQuery(uri, projection, selection, urimId);
            case URIM_PRACTICES_COUNT:
        		Log.d(LOGTAG, "privder query() 22 practices count");
        		return getMetaQuery(uri, projection, selection, urimId);
            case URIM_PRACTICES_PREV_ID:
            	Log.d(LOGTAG, "privder query() 23a practices prev");
            	return getMetaQuery(uri, projection, selection, urimId);
            case URIM_PRACTICES_NEXT_ID:
            	Log.d(LOGTAG, "privder query() 23b practices next");
            	return getMetaQuery(uri, projection, selection, urimId);
            case URIM_INSTRUMENTS:
        		Log.d(LOGTAG, "privder query() 24 general");
        		return getGeneralQuery(uri, projection, selection, urimId);
            case URIM_GOALS:
        		Log.d(LOGTAG, "privder query() 30");
        		return getGeneralQuery(uri, projection, selection, urimId);
            default:
        		Log.d(LOGTAG, "privder query() 40");
                throw new IllegalArgumentException("query() Unknown Uri: " + uri);
        } // END switch(urimId)
	} // END query()

	@Override
	public String getType(Uri uri) {
		// TODO Auto-generated method stub
		// vnd.android.cursor.dir/vnd.com.example.provider.table1 # muti-row return
		// vnd.android.cursor.item/vnd.com.example.provider.table1 # single row return
		// FIXME: later - implement getStreamTypes() for file support (if that's even required -- we may access files directly for now...)
		return null;
	}

	/**
	 * General insert handler, adding complete rows of data to Sessions DB tables.
	 */
	@Override
	public Uri insert(Uri uri, ContentValues values) {
		Log.d(LOGTAG, "privder insert() 00 uri: " + uri + " :values: " + values);
		
		SQLiteDatabase db = mDBHelper.getWritableDatabase();
		long rowId = -1;	// set rowId to a value that represents invalid/unknown data.
		Uri newUri;

        // Use the UriMatcher to see what kind of query we have and format the db query accordingly
        switch (sUriMatcher.match(uri)) {
            case URIM_GOALS:
            	// FIXME: validate data (fields, content) before putting it into the DB
                rowId = db.insert(SessionsDB.GOALS_TABLE, null, values);
                newUri = SessionsDB.CONTENT_GOALS_URI;
            	break;
            case URIM_INSTRUMENTS:
            	// FIXME: validate data (fields, content) before putting it into the DB
                rowId = db.insert(SessionsDB.INSTRUMENTS_TABLE, null, values);
                newUri = SessionsDB.CONTENT_INSTRUMENTS_URI;
                break;
            case URIM_PRACTICES:
            	// FIXME: validate data...
            	rowId = db.insert(SessionsDB.PRACTICES_TABLE, null, values);
            	newUri = SessionsDB.CONTENT_PRACTICES_URI;
            	break;
            default:
        		Log.d(LOGTAG, "privder query() 40");
                throw new IllegalArgumentException("insert() Unknown Uri: " + uri);
        } // END switch(sUriMatcher)
        
        if (rowId > 0) {
        	// notify of the table change, using the generic newUri
    		Log.d(LOGTAG, "privder query() 50 newUri:" + newUri);
        	getContext().getContentResolver().notifyChange(newUri, null);
        	// update the newUri to include the rowId, and return it
        	newUri = ContentUris.withAppendedId(newUri, rowId);
    		Log.d(LOGTAG, "privder query() 54 newUri:" + newUri);
            return newUri;
        }
        throw new SQLException("Failed to insert row into " + uri);
	} // END insert()

	@Override
	public int update(Uri uri, ContentValues values, String selection,
			String[] selectionArgs) {		
		Log.d(LOGTAG, "privder update() 00 uri: " + uri + " :values: " + values.toString() + ":selection:" + selection);
		
		SQLiteDatabase db = mDBHelper.getWritableDatabase();
		int rowCount = 0;  // Number of rows updated.
		
        // Use the UriMatcher to see what will be deleted and format the db query accordingly
        switch (sUriMatcher.match(uri)) {
            case URIM_PRACTICES_ID:
            	// Create where clause matching ID; append to selection if present.
            	long lid = ContentUris.parseId(uri);
            	String lsel = "( _ID = " + lid + " )";
            	if ( (null != selection) && (selection.length() > 0) ) {
            		selection = selection + " AND " + lsel;
            	} else {
            		selection = lsel;
            	}
        		Log.d(LOGTAG, "privder update() 30   PRACT ID sel: " + selection);
            	rowCount = rowCount + db.update(SessionsDB.PRACTICES_TABLE, values, selection, selectionArgs);             	
            	break;
            default:
        		Log.d(LOGTAG, "privder update() 40");
                throw new IllegalArgumentException("delete() Un-Handled Uri: " + uri);
        } // END switch(sUriMatcher)
        
        if (rowCount > 0) {
    		Log.d(LOGTAG, "privder update() 50 rowCount:" + rowCount);
        	// notify of the table change, using the generic newUri
        	getContext().getContentResolver().notifyChange(uri, null); // FIXME: confirm if we need to notify on the URI with _ID, without _ID, or both...
        }		
		return rowCount;
	}
	
	@Override
	public int delete(Uri uri, String selection, String[] selectionArgs) {
		Log.d(LOGTAG, "privder delete() 00 uri: " + uri + " :selection: " + selection);
						
		SQLiteDatabase db = mDBHelper.getWritableDatabase();
		int rowCount = 0;  // Number of rows deleted.
		
        // Use the UriMatcher to see what will be deleted and format the db query accordingly
        switch (sUriMatcher.match(uri)) {
            case URIM_GOALS:
                rowCount = rowCount + db.delete(SessionsDB.GOALS_TABLE, selection, selectionArgs);
            	break;
            case URIM_INSTRUMENTS:
                rowCount = rowCount + db.delete(SessionsDB.INSTRUMENTS_TABLE, selection, selectionArgs);
                break;
            case URIM_PRACTICES:
        		Log.d(LOGTAG, "privder delete() 20   PRACT");
            	rowCount = rowCount + db.delete(SessionsDB.PRACTICES_TABLE, selection, selectionArgs);
            	break;
            case URIM_PRACTICES_ID:
            	// Create where clause matching ID; append to selection if present.
            	long lid = ContentUris.parseId(uri);
            	String lsel = "( _ID = " + lid + " )";
            	if ( (null != selection) && (selection.length() > 0) ) {
            		selection = selection + " AND " + lsel;
            	} else {
            		selection = lsel;
            	}
        		Log.d(LOGTAG, "privder delete() 30   PRACT ID sel: " + selection);
            	rowCount = rowCount + db.delete(SessionsDB.PRACTICES_TABLE, selection, selectionArgs);             	
            	break;
            default:
        		Log.d(LOGTAG, "privder delete() 40");
                throw new IllegalArgumentException("delete() Unknown Uri: " + uri);
        } // END switch(sUriMatcher)
        
        if (rowCount > 0) {
    		Log.d(LOGTAG, "privder delete() 50 rowCount:" + rowCount);
        	// notify of the table change, using the generic newUri
        	getContext().getContentResolver().notifyChange(uri, null);
        }		
		return rowCount;
	} // END delete()

	// Helper private methods
    /**
     * A general method to handle select queries, with the goal of code re-use.
     * @param uri Uri corresponding with the query
     * @param projection String[] array of strings table columns to query 
     * @param selection String with selection filter
     * @param urimId integer corresponding with the UriMatch of uri (to save resources)
     * @return Cursor with query results which is set On to monitor for notifications 
     */
    private Cursor getGeneralQuery(Uri uri, String[] projection, String selection, int urimId) {
		Log.d(LOGTAG, "privder getGeneralQuery() 00 uri: " + uri);
        SQLiteQueryBuilder builder = new SQLiteQueryBuilder();

        switch (urimId) {
        	case URIM_INSTRUMENTS:
                builder.setTables(SessionsDB.INSTRUMENTS_TABLE);
        		Log.d(LOGTAG, "privder getGeneralQuery() 11");
        		break;
        	case URIM_GOALS:
        		builder.setTables(SessionsDB.GOALS_TABLE);
        		Log.d(LOGTAG, "privder getGeneralQuery() 13");
        		break;
        	case URIM_PRACTICES:
        	case URIM_PRACTICES_ID:
        		builder.setTables(SessionsDB.PRACTICES_TABLE);
        		Log.d(LOGTAG, "privder getGeneralQuery() 15");
        		if (URIM_PRACTICES_ID == urimId) {
        			long lrow = ContentUris.parseId(uri);
        			selection = " _ID = " + lrow;
        		}
        		break;
            default:
        		Log.d(LOGTAG, "privder getGeneralQuery() 19");
                throw new IllegalArgumentException("getGeneralQuery() Unknown Uri: " + uri);
        } // END switch(urimId)
        
		Log.d(LOGTAG, "privder getGeneralQuery() 30");
		
		String lsql = builder.buildQuery(projection, selection, null, null, null, null, null);
		Log.d(LOGTAG, "privder getGeneralQuery() 41 SQL buildQuery:" + lsql);
        Cursor cursor = builder.query(mDBHelper.getReadableDatabase(),
                projection, selection, null, null, null, null);
        if (cursor == null) {
    		Log.d(LOGTAG, "privder getGeneralQuery() 51 no cursor - return null");
            return null;
        } else if (!cursor.moveToFirst()) {
    		Log.d(LOGTAG, "privder getGeneralQuery() 54 unable to get to first row - return null");
            cursor.close();
            return null;
        }
		// Set to check for table changes, so cursor is "auto"updated.
		cursor.setNotificationUri(getContext().getContentResolver(), uri);
		Log.d(LOGTAG, "privder getGeneralQuery() 99 uri: " + uri);
		return cursor;
    } // END getGeneralQuery()
    
    
    
	// Helper private methods
    /**
	 * Meta queries, that return information about the table rather than
	 * content data from the table. Includes table count, next and previous row
	 * _ID, and min and max row _ID.
     * 
     * @param uri Uri corresponding with the query
     * @param projection String[] array of strings table columns to query 
     * @param selection String with selection filter
     * @param urimId integer corresponding with the UriMatch of uri (to save resources)
     * @return Cursor with query results which is set On to monitor for notifications 
     */
    private Cursor getMetaQuery(Uri uri, String[] projection, String selection, int urimId) {
		Log.d(LOGTAG, "privder getMetaQuery() 00 uri: " + uri);
		String lsql;
		long lrow;
		String [] largs = null;
		SQLiteDatabase db = mDBHelper.getWritableDatabase();
		
		switch (urimId) {
       	case URIM_PRACTICES_COUNT:
    		lsql = "select count(*), min(_ID), max(_ID) from " + SessionsDB.PRACTICES_TABLE; 
       		Log.d(LOGTAG, "privder getMetaQuery() 15   lsql: " + lsql);
       		break;
       	case URIM_PRACTICES_PREV_ID:
			lrow = ContentUris.parseId(uri);
			largs = new String [] { String.valueOf(lrow) };
    		lsql = "select max(_ID) from " + SessionsDB.PRACTICES_TABLE + " where ( _ID < ? ) ";
       		Log.d(LOGTAG, "privder getMetaQuery() 16   lsql: " + lsql);
       		break;
       	case URIM_PRACTICES_NEXT_ID:
			lrow = ContentUris.parseId(uri);
			largs = new String [] { String.valueOf(lrow) };
    		lsql = "select min(_ID) from " + SessionsDB.PRACTICES_TABLE + " where ( _ID > ? ) ";
       		Log.d(LOGTAG, "privder getMetaQuery() 17   lsql: " + lsql);
       		break;
       default:
    	   Log.d(LOGTAG, "privder getMetaQuery() 19 - crashing - unknown uri: " + uri);
           throw new IllegalArgumentException("getCountQuery() Unknown Uri: " + uri);
       } // END switch(urimId)       
		Cursor cursor= db.rawQuery(lsql, largs);
        if (cursor == null) {
    		Log.d(LOGTAG, "privder getMetaQuery() 51 no cursor - return null");
            return null;
        } else if (!cursor.moveToFirst()) {
    		Log.d(LOGTAG, "privder getMetaQuery() 54 unable to get to first row - return null");
            cursor.close();
            return null;
        }
	return cursor;
    } // END getMetaQuery()


}
